@@ -42,6 +42,7 @@ bootdata.trans.msm <- function(x) {
     inds <- sample(which(duplicated(dat[,un["subject"]], fromLast=TRUE)), replace=TRUE)
     ## make new data by interleaving corresponding "from-state" and "to-state" rows
     ntrans <- length(inds)
+    dat <- dat[,-match(un["subject"], names(dat)),drop=FALSE]
     z <- array(c(unlist(dat[inds,]), # "from-state" data
                  unlist(dat[inds+1,])), # "to-state" data
                dim=c(ntrans, ncol(dat), 2))
@@ -50,7 +51,10 @@ bootdata.trans.msm <- function(x) {
     data.boot <- as.data.frame(data.boot)
     ## label every transition in new data as from a different subject
     data.boot[,un["subject"]] <- rep(1:ntrans, each=2)
-    for (i in which(sapply(data.boot, is.factor)))
+    ## make sure factor covariates retain their original level labels
+    facvars <- names(dat)[sapply(dat, is.factor)] 
+    faccovs <- setdiff(facvars, un)
+    for (i in faccovs)
         data.boot[,i] <- factor(data.boot[,i], labels=sort(unique(dat[,i])))
     data.boot
 }
@@ -253,7 +257,7 @@ normboot.msm <- function(x, stat, B=1000) {
     params <- matrix(nrow=B, ncol=x$paramdata$npars)  # replicate constrained parameters.
     params[,x$paramdata$optpars] <- sim
     params[,x$paramdata$fixedpars] <- rep(x$paramdata$params[x$paramdata$fixedpars], each=B)
-    params[,x$paramdata$hmmpars] <- rep(msm.mninvlogit.transform(x$paramdata$params[x$paramdata$hmmpars], x$hmodel$plabs, x$hmodel$parstate), each=B)
+    params[,x$paramdata$hmmpars] <- rep(msm.mninvlogit.transform(x$paramdata$params[x$paramdata$hmmpars], x$hmodel), each=B)
     params <- params[, !duplicated(abs(x$paramdata$constr)), drop=FALSE][, abs(x$paramdata$constr), drop=FALSE] *
         rep(sign(x$paramdata$constr), each=B)
 
