@@ -33,7 +33,7 @@ msm.check.hcovariates <- function(hcovariates, qmodel)
         stop("hcovariates of length ", length(hcovariates), ", expected ", qmodel$nstates)
       for (i in hcovariates) {
           if (!is.null(i) )
-            if ( class(i) != "formula") stop("hcovariates should be a list of formulae or NULLs")
+            if (!inherits(i, "formula")) stop("hcovariates should be a list of formulae or NULLs")
       }
   }
 
@@ -423,7 +423,7 @@ msm.form.hranges <- function(ranges, hmodel)
                             " less than theoretical maximum of ", ran.default[j,2], ", ignoring")
                     ran.user[j,2] <- ran.default[j,2]
                 }
-                hranges[rownames(hranges)==i,][j,] <- ran.user[j,]
+                hranges[which(rownames(hranges)==i)[j],] <- ran.user[j,]
             }
         }
     }
